#ifndef _TransparentWidget_h_
#define _TransparentWidget_h_
// $Id: TransparentWidget.h,v 1.30 2012/05/04 10:16:34 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    TransparentWidget.h
\brief   transparent widget
\author  Hugo Pereira
\version $Revision: 1.30 $
\date    $Date: 2012/05/04 10:16:34 $
*/

#include "Counter.h"

#include <QtGui/QAction>
#include <QtGui/QMoveEvent>
#include <QtGui/QPaintEvent>
#include <QtGui/QResizeEvent>
#include <QtGui/QShowEvent>
#include <QtGui/QWidget>

namespace TRANSPARENCY
{

    //! transparent widget
    class TransparentWidget: public QWidget, public Counter
    {

        //! Qt meta object declaration
        Q_OBJECT

        public:

        //! constructor
        TransparentWidget( QWidget *parent = 0, Qt::WindowFlags flags = 0 );

        //!@name actions
        //@{

        //! update background
        QAction& updateBackgroundAction( void ) const
        { return *updateBackgroundAction_; }

        //! reload background action
        QAction& reloadBackgroundAction( void ) const
        { return *reloadBackgroundAction_; }

        //! reload blur region action
        QAction& reloadBlurRegionAction( void ) const
        { return *reloadBlurRegionAction_; }

        //! inverse colors
        QAction& inverseColorsAction( void ) const
        { return *inverseColorsAction_; }

        //@}

        //! window opacity
        /*! this overloads the QWidget method to hack for WIN system */
        void setWindowOpacity( double );

        //! background changed
        void setBackgroundChanged( const bool& value )
        { backgroundChanged_ = value; }

        //! foreground
        virtual const QColor& foregroundColor( void ) const
        { return inverseColorsAction().isChecked() ? shadowColor_ : foregroundColor_; }

        //! shadow
        virtual const QColor& shadowColor( void ) const
        { return inverseColorsAction().isChecked() ? foregroundColor_ : shadowColor_; }

        //! foreground intensity
        virtual int foregroundIntensity( void ) const
        { return foregroundIntensity_; }

        //! shadow offset
        virtual int shadowOffset( void ) const
        { return shadowOffset_; }

        public slots:

        //! force reloading of the background
        virtual void setBackgroundChanged( void );

        protected:

        //! enable/disable transparency
        virtual void _setTransparent( const bool& value )
        {
            if( value == transparent_ ) return;
            transparent_ = value;
            setBackgroundChanged( true );
        }

        //! enable/disable transparency
        virtual void _setBlurEnabled( const bool& value )
        {
            if( value == blurEnabled_ ) return;
            blurEnabled_ = value;
            setBackgroundChanged( true );
        }

        //! transparency
        virtual const bool& _transparent( void ) const
        { return transparent_; }

        //! foreground
        virtual void _setForegroundColor( const QColor& );

        //! shadow
        virtual void _setShadowColor( const QColor& );

        //! tint
        virtual void _setTintColor( const QColor& );

        //! tint color
        virtual const QColor& _tintColor( void ) const
        { return tintColor_; }

        //! highlight
        virtual void _setHighlightColor( const QColor& );

        //! highlight color
        virtual const QColor& _highlightColor( void ) const
        { return highlightColor_; }

        //! shadow offset
        virtual void _setShadowOffset( int value )
        { shadowOffset_ = value; }

        //! foreground intensity
        virtual void _setForegroundIntensity( int value );

        //! highlighted
        virtual void _setHighlighted( bool value )
        { highlighted_ = value; }

        //! highlighted
        virtual const bool& _highlighted( void ) const
        { return highlighted_; }

        //! background pixmap
        virtual QPixmap& _backgroundPixmap( void )
        { return backgroundPixmap_; }

        //! background pixmap
        virtual const QPixmap& _backgroundPixmap( void ) const
        { return backgroundPixmap_; }

        //! opacity
        const double& _opacity( void ) const
        { return opacity_; }

        //!@name event handlers
        //@{

        //! move
        virtual void moveEvent( QMoveEvent* );

        //! resize
        virtual void resizeEvent( QResizeEvent* );

        //! show
        virtual void showEvent( QShowEvent* );

        //! enter event
        virtual void enterEvent( QEvent *event );

        //! leave event
        virtual void leaveEvent( QEvent *event );

        //! paint
        virtual void paintEvent( QPaintEvent* );

        //@}

        //! background changed
        const bool& _backgroundChanged( void ) const
        { return backgroundChanged_; }

        //! paint background on devide
        virtual void _paintBackground( QPaintDevice&, const QRect& );

        //! paint main widget on devide
        /*! this must be re-implemented by derived classes */
        virtual void _paint( QPaintDevice&, const QRect& )
        { return; }

        //! true if blur is enabled
        virtual bool _blurEnabled( void ) const
        { return blurEnabled_; }

        //! update blur region
        virtual void _updateBlurRegion( const QRegion& );

        protected slots:

        //! reload background
        virtual void _reloadBackground( void );

        //! update background pixmap
        virtual void _updateBackgroundPixmap( void );

        //! update blur region
        virtual void _updateBlurRegion( void )
        {
            if( blurRegion_.isEmpty() ) blurRegion_ = rect().translated( mapTo( window(), QPoint(0,0) ) );
            _updateBlurRegion( blurRegion_ );
        }

        //! toggle inverse colors
        virtual void _toggleInverseColors( bool );

        private slots:

        //! update configuration
        void _updateConfiguration( void );

        private:

        //! actions
        void _installActions( void );

        //!@name actions
        //@{

        //! update background
        QAction *updateBackgroundAction_;

        //! reload background
        QAction* reloadBackgroundAction_;

        //! reload blur region
        QAction* reloadBlurRegionAction_;

        //! inverse colors
        QAction* inverseColorsAction_;

        //@}

        //! transparency enabled
        bool transparent_;

        //! true when background needs to be reloaded
        bool backgroundChanged_;

        //! foreground
        QColor foregroundColor_;

        //! shadow
        QColor shadowColor_;

        //! tint color
        QColor tintColor_;

        //! foreground intensity
        int foregroundIntensity_;

        //! shadow offset
        int shadowOffset_;

        //! true when pointer is in window
        bool highlighted_;

        //! true when blur is enabled
        bool blurEnabled_;

        //! store last blur region
        QRegion blurRegion_;

        //! window opacity
        double opacity_;

        //! tint color
        QColor highlightColor_;

        #if defined(Q_WS_WIN)
        //! widget pixmap
        /*! it is used as widget storage when using full translucency */
        QPixmap widgetPixmap_;
        #endif

        //! background pixmap
        QPixmap backgroundPixmap_;

    };

};

#endif
