// $Id: ConfigurationDialog.cpp,v 1.34 2012/07/18 15:51:54 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "ConfigurationDialog.h"

#include "CustomToolBar.h"
#include "Debug.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "Icons.h"
#include "OptionCheckBox.h"
#include "OptionColorDisplay.h"
#include "OptionRadioButton.h"
#include "OptionSlider.h"
#include "OptionSpinBox.h"

#include "SvgConfiguration.h"
#include "TransparencyConfiguration.h"

#include <QtGui/QButtonGroup>
#include <QtGui/QGroupBox>
#include <QtGui/QLabel>
#include <QtGui/QLayout>
#include <QtGui/QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
    BaseConfigurationDialog( parent )
{

    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( "Configuration - Taskbar" );

    // base configuration
    baseConfiguration( 0 );

    // transparency configuration
    QWidget* page;
    TRANSPARENCY::TransparencyConfiguration* transparency_configuration;

    page = &addPage( IconEngine::get( ICONS::PREFERENCE_TRANSPARENCY ), "Transparency", "Window transparency settings" );
    page->layout()->addWidget( transparency_configuration = new TRANSPARENCY::TransparencyConfiguration( page ));
    addOptionWidget( transparency_configuration );

    // generic widgets
    QGroupBox *box;
    GridLayout* gridLayout;
    OptionSpinBox* spinbox;
    OptionCheckBox* checkBox;
    OptionSlider* slider;

    // svg background
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_BACKGROUND ), "Background", "Window background settings" );
    SVG::SvgConfiguration* svg_configuration( new SVG::SvgConfiguration( page ) );
    page->layout()->addWidget( svg_configuration);
    addOptionWidget( svg_configuration );

    // buttons
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_APPEARANCE ), "Appearance", "Window appearance" );
    box = new QGroupBox( "Button size", page );
    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    box->setLayout( gridLayout );
    page->layout()->addWidget( box );

    gridLayout->addWidget( new QLabel( "Margin:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "MARGIN" ) );
    spinbox->setSuffix( "px" );
    spinbox->setToolTip( "Space between buttons and window border" );
    addOptionWidget( spinbox );

    gridLayout->addWidget( new QLabel( "Spacing:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "SPACING" ) );
    spinbox->setSuffix( "px" );
    spinbox->setToolTip( "Space between consecutive buttons" );
    addOptionWidget( spinbox );

    gridLayout->addWidget( new QLabel( "Minimum size:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "MIN_BUTTON_SIZE" ) );
    spinbox->setToolTip( "Button minimum size (pixels)" );
    spinbox->setSuffix( "px" );
    spinbox->setMinimum(0);
    spinbox->setMaximum(128);
    addOptionWidget( spinbox );

    gridLayout->addWidget( new QLabel( "Maximum size:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "MAX_BUTTON_SIZE" ) );
    spinbox->setToolTip( "Button maximum size (pixels)" );
    spinbox->setSuffix( "px" );
    spinbox->setMinimum(0);
    spinbox->setMaximum(256);
    addOptionWidget( spinbox );

    // button highlight
    box = new QGroupBox( "Highlight", page );
    box->setLayout( new QVBoxLayout() );
    box->layout()->setMargin(5);
    box->layout()->setSpacing(5);
    page->layout()->addWidget( box );

    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    box->layout()->addItem( hLayout );

    QButtonGroup* button_group( new QButtonGroup( this ) );
    button_group->setExclusive( true );

    OptionRadioButton* radio_button;
    hLayout->addWidget( radio_button = new OptionRadioButton( "none", box, "NO_HIGHLIGHT" ) );
    radio_button->setToolTip( "Do not highlight buttons" );
    addOptionWidget( radio_button );
    button_group->addButton( radio_button );

    hLayout->addWidget( radio_button = new OptionRadioButton( "On enter", box, "HIGHLIGHT_ON_ENTER" ) );
    radio_button->setToolTip( "Highlight buttons when hovered" );
    addOptionWidget( radio_button );
    button_group->addButton( radio_button );

    hLayout->addWidget( radio_button = new OptionRadioButton( "On click", box, "HIGHLIGHT_ON_CLICK" ) );
    radio_button->setToolTip( "Highlight buttons when clicked" );
    addOptionWidget( radio_button );
    button_group->addButton( radio_button );

    hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    box->layout()->addItem( hLayout );

    hLayout->addWidget( new QLabel( "Intensity:", box ) );
    hLayout->addWidget( slider = new OptionSlider( box, "HIGHLIGHT_INTENSITY" ) );
    slider->setScale( 100 );
    slider->setRange( 0, 100 );
    slider->setToolTip( "Button highlight intensity." );
    addOptionWidget( slider );

    // tooltips
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_UNSORTED ), "Unsorted", "Additional unsorted settings" );
    box = new QGroupBox( "Tooltips", page );
    box->setLayout( new QVBoxLayout() );
    box->layout()->setMargin(5);
    box->layout()->setSpacing(5);
    page->layout()->addWidget( box );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(0);
    gridLayout->setMaxCount(2);
    box->layout()->addItem( gridLayout );

    gridLayout->addWidget( new QLabel( "Tooltip size:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "TOOLTIP_HEIGHT" ) );
    spinbox->setSuffix( "px" );
    spinbox->setMinimum(0);
    spinbox->setMaximum(1024);
    spinbox->setToolTip( "Tooltip window size (pixels)" );
    addOptionWidget( spinbox );

    gridLayout->addWidget( new QLabel( "Tooltip delay:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "TOOLTIP_DELAY" ) );
    spinbox->setSuffix( "ms" );
    spinbox->setMinimum(0);
    spinbox->setMaximum(1024);
    spinbox->setToolTip( "Delay before the tooltip is shown" );
    addOptionWidget( spinbox );

    box->layout()->addWidget( checkBox = new OptionCheckBox( "Show tooltips", box, "SHOW_TOOLTIP" ) );
    checkBox->setToolTip( "Show tooltips" );
    addOptionWidget( checkBox );

    OptionCheckBox* styleCheckBox;
    box->layout()->addWidget( styleCheckBox = new OptionCheckBox( "Use widget style", box, "USE_STYLED_TOOLTIPS" ) );
    addOptionWidget( styleCheckBox );

    box->layout()->addWidget( checkBox = new OptionCheckBox( "Round tooltips", box, "ROUND_TOOLTIP" ) );
    checkBox->setToolTip( "Round tooltips" );
    checkBox->setEnabled( false );
    addOptionWidget( checkBox );

    connect( styleCheckBox, SIGNAL( toggled( bool ) ), checkBox, SLOT( setDisabled( bool ) ) );

    box->layout()->addWidget( checkBox = new OptionCheckBox( "Use transparency", box, "TRANSPARENT_TOOLTIP" ) );
    checkBox->setToolTip( "Transparent tooltips" );
    checkBox->setEnabled( false );
    addOptionWidget( checkBox );

    connect( styleCheckBox, SIGNAL( toggled( bool ) ), checkBox, SLOT( setDisabled( bool ) ) );

    // misc
    box = new QGroupBox( "Unsorted", page );
    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    box->setLayout( gridLayout );
    page->layout()->addWidget( box );

    gridLayout->addWidget( new QLabel( "Displacement grid size:", box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "GRID" ) );
    spinbox->setSuffix( "px" );
    spinbox->setMinimum(1);
    spinbox->setToolTip( "Step size when moving the taskbar on desktop" );
    addOptionWidget( spinbox );

    // load initial configuration
    read();

}
