// $Id: TaskBarMinimum.cpp,v 1.73 2012/09/10 18:42:13 hpereira Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is_ distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    TaskBarMinimum.cc
\brief   transparent frame to display TaskButton(s)
\author  Hugo Pereira
\version $Revision: 1.73 $
\date    $Date: 2012/09/10 18:42:13 $
*/

#include "TaskBarMinimum.h"
#include "RoundedRegion.h"
#include "SvgEngine.h"
#include "TaskToolTip.h"
#include "XmlOptions.h"

#include "CompositeEngine.h"

#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>

//____________________________________________________
TaskBarMinimum::TaskBarMinimum( QWidget* parent ):
    TaskBarBase( parent ),
    rootPixmapChanged_( true )
{ Debug::Throw( "TaskBarMinimum::TaskBarMinimum.\n" ); }

//____________________________________________________
void TaskBarMinimum::_updateRootPixmap( void )
{

    Debug::Throw( "TaskBarMinimum::_updateRootPixmap" );
    _setRootPixmapChanged( false );
    if( Debug::level() >= 1 )
    {
        // solid background
        _setRootPixmap( QPixmap( size() ) );
        _rootPixmap().fill( palette().color( backgroundRole() ) );

    } else if( TRANSPARENCY::CompositeEngine::get().isEnabled() ) {

        _setRootPixmap( QPixmap( size() ) );
        QPainter painter( &_rootPixmap() );
        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
        painter.fillRect( _rootPixmap().rect(), Qt::transparent);

    } else {

        _setRootPixmap( QPixmap::grabWindow(
            QApplication::desktop()->winId(),
            geometry().x(), geometry().y(),
            geometry().width(), geometry().height() ) );

    }
}

//___________________________________________________
void TaskBarMinimum::_updateBackgroundPixmap( void )
{
    Debug::Throw( "TaskBarMinimum::TaskBarFixed.\n" );
    if( _rootPixmapChanged() )
    {
        // there is no need to update the root pixmap when composite engine is off,
        // unless during resize events, for which the method is called directly
        if( !TRANSPARENCY::CompositeEngine::get().isEnabled() ) _updateRootPixmap();
        else _setRootPixmapChanged( false );
    }

    TaskBarBase::_updateBackgroundPixmap();

    // need to draw svg on background pixmap if button list is empty
    // due to the way empty taskbars are implemented
    if( _buttons().empty() &&_useSvg() && SVG::SvgEngine::get().isValid() )
    {
        QPainter painter( &_backgroundPixmap() );
        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
        painter.drawPixmap( QPoint(0,0),  SVG::SvgEngine::get().get( size() ) );
    }

}

//______________________________________________
void TaskBarMinimum::resizeEvent( QResizeEvent* event )
{
    _updateRootPixmap();

    /*
    need to bypass TaskBarBase::resizeEvent, in order not to have
    the entire background blurred, when enabled
    */
    if( attributes().flag( TaskBarAttributes::ROUND_CORNERS ) ) _roundCorners();
    TransparentWidget::resizeEvent( event );
}

//______________________________________________________
void TaskBarMinimum::_resize( void )
{

    Debug::Throw( "TaskBarMinimum::_resize.\n" );

    // if no tasks, set empty message and exit
    if( _buttons().empty() ) {
        resize( emptySize() );
        return;
    }

    // dimensions
    _setDimensions();
    _updateMinimumSize();
    _updateMaximumSize();
    _updateMinimumRectPosition();

    // svg
    if( _useSvg() && SVG::SvgEngine::get().isValid() ) _preloadSvg();

    // make sure taskbar is in minimum state
    // resize
    _setRootPixmapChanged( true );
    _setAction( MINIMUM );
    resize( _maximumSize() );

}

//___________________________________________________
void TaskBarMinimum::_placeButtons( void )
{

    Debug::Throw( "TaskBarMinimum::_placeButtons.\n" );

    // check that there is at least one button
    if( _buttons().empty() ) return;

    // starting positions
    int x(0);
    int y(0);

    const TaskBarAttributes::Layout &layout( attributes().layout() );

    if( layout & ( TaskBarAttributes::VERTICAL | TaskBarAttributes::HORIZONTAL ) )
    {

        foreach( TaskButton* button, _buttons() )
        {

            button->setRect( QPoint(x,y), QSize( button->currentSize().width() + _spacing(), button->currentSize().height() + _spacing()  ) );

            // update to next button
            if( layout & TaskBarAttributes::VERTICAL ) y+= button->rect().height();
            else if( layout & TaskBarAttributes::HORIZONTAL )  x += button->rect().width();
        }

    } else if( layout & TaskBarAttributes::BOX ) {

        int columnIndex(0);
        int lineIndex(0);

        // compute column width/line height
        QVector<int> columnWidth( _numColumns(), 0 );
        QVector<int> lineHeight( _numLines(), 0 );
        foreach( TaskButton* button, _buttons() )
        {

            lineHeight[lineIndex] = std::max( lineHeight[lineIndex], button->currentSize().height() + _spacing() );
            columnWidth[columnIndex] = std::max( columnWidth[columnIndex], button->currentSize().width() + _spacing() );
            columnIndex++;
            if( columnIndex >= _numColumns() ) {
                columnIndex = 0;
                lineIndex++;
            }

        }

        // place buttons
        columnIndex = 0;
        lineIndex = 0;
        foreach( TaskButton* button, _buttons() )
        {
            button->setRect( QPoint( x, y ), QSize( columnWidth[columnIndex], lineHeight[lineIndex] ) );
            x+=columnWidth[columnIndex];
            columnIndex++;
            if( columnIndex >= _numColumns() ) {
                x = 0;
                columnIndex = 0;

                y+= lineHeight[lineIndex];
                lineIndex++;
            }
        }
    }

}

//______________________________________________________
void TaskBarMinimum::_addOffsets( void )
{

    // dump
    Debug::Throw() << "TaskBarMinimum::_addOffsets -"
        << " size: (" << width() << "," << height() << ")"
        << endl;

    // get bounding rect for current button sizes
    QRect boundingRect;
    foreach( TaskButton* button, _buttons() )
    {
        boundingRect.setWidth( std::max( boundingRect.width(), button->rect().right() ) );
        boundingRect.setHeight( std::max( boundingRect.height(), button->rect().bottom() ) );
    }

    // compute offsets
    QPoint offset( (size().width() - boundingRect.width())/2, (size().height() - boundingRect.height())/2 );
    const TaskBarAttributes::Layout &layout( attributes().layout() );

    // add offsets
    foreach( TaskButton* button, _buttons() )
    {
        QPoint button_offset( offset );
        if( layout & TaskBarAttributes::VERTICAL )
        {

            if( layout & TaskBarAttributes::TOP_LEFT ) button_offset.setX(_margin());
            else if( layout & TaskBarAttributes::BOTTOM_RIGHT ) button_offset.setX( width() - button->rect().width() - _margin() );
            else if( layout & TaskBarAttributes::CENTER ) button_offset.setX( (width() - button->rect().width())/2 );

        } else if( layout & TaskBarAttributes::HORIZONTAL ) {

            if( layout & TaskBarAttributes::TOP_LEFT ) button_offset.setY(_margin());
            else if( layout & TaskBarAttributes::BOTTOM_RIGHT ) button_offset.setY( height() - button->rect().height() - _margin() );
            else if( layout & TaskBarAttributes::CENTER ) button_offset.setY( (height() - button->rect().height())/2 );

        }

        button->setRect( button->rect().translated( button_offset ) );

    }

}

//___________________________________________________
bool TaskBarMinimum::_drawButtons( QPaintDevice& device, QRect rect )
{

    // if no tasks, do nothing
    if( _buttons().empty() ) return false;

    // place the buttons
    _placeButtons();
    _addOffsets();

    // redraw
    QPainter painter( &device );
    painter.setClipRect( rect );

    if( TRANSPARENCY::CompositeEngine::get().isEnabled() )
    {
        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
    }

    // draw root pixmap into entire window
    painter.drawPixmap( 0, 0, _rootPixmap() );

    // draw background pixmap clipped to visibleRect
    _updateVisibleRect();
    _drawVisibleRect( painter );

    // draw buttons, translated by offset if needed
    painter.setCompositionMode(QPainter::CompositionMode_SourceOver );
    painter.translate( _offset() );

    bool out( false );
    foreach( TaskButton* button, _buttons() )
    {

        button->paint( painter );
        if( _action() != ENTER )
        {
            button->updateDirection();
            button->updateHighlightDirection();
            out |= (button->direction() != TaskButton::NONE);
            out |= button->highlightDirection() != TaskButton::NOHIGHLIGHT;
        }

    }

    painter.end();

    return out;

}

//___________________________________________________
void TaskBarMinimum::_drawVisibleRect( QPainter& painter )
{
    painter.save();

    if( _useSvg() && SVG::SvgEngine::get().isValid() )
    {

        painter.translate( _visibleRect().topLeft() );
        painter.drawPixmap( QPoint(0,0), SVG::SvgEngine::get().get( _visibleRect().size() ) );

    } else {
        if( attributes().flag( TaskBarAttributes::ROUND_CORNERS ) ) painter.setClipRegion( RoundedRegion( _visibleRect(), RoundedRegion::ALL ) );
        else painter.setClipRegion( _visibleRect() );
        painter.drawPixmap( 0, 0, _backgroundPixmap() );
    }
    painter.restore();
}

//_____________________________________________________________
void TaskBarMinimum::_updateMinimumSize( void )
{

    Debug::Throw( "TaskBarMinimum::_updateMinimumSize.\n" );

    // compute minimum size
    QSize minimumSize;

    // compute visible rect
    const TaskBarAttributes::Layout& layout ( attributes().layout() );
    if( layout & TaskBarAttributes::VERTICAL )
    {
        foreach( TaskButton* button, _buttons() )
        {
            minimumSize.setHeight( minimumSize.height() + button->minimumSize().height() + _spacing() );
            minimumSize.setWidth( std::max<int>( minimumSize.width(), button->minimumSize().width() + _spacing() + 2*_margin() ) );
        }

        minimumSize.setHeight( minimumSize.height() + 2*_margin() );

    } else if( layout & TaskBarAttributes::HORIZONTAL ) {

        foreach( TaskButton* button, _buttons() )
        {
            minimumSize.setWidth( minimumSize.width() + button->minimumSize().width() + _spacing() );
            minimumSize.setHeight( std::max<int>( minimumSize.height(), button->minimumSize().height() + _spacing() + 2*_margin() ) );
        }

        minimumSize.setWidth( minimumSize.width() + 2*_margin() );

    } else {

        int columnIndex(0);
        int lineIndex(0);

        // compute column width/line height
        QVector<int> columnWidth( _numColumns(), 0 );
        QVector<int> lineHeight( _numLines(), 0 );
        foreach( TaskButton* button, _buttons() )
        {

            lineHeight[lineIndex] = std::max( lineHeight[lineIndex], button->currentSize().height() + _spacing() );
            columnWidth[columnIndex] = std::max( columnWidth[columnIndex], button->currentSize().width() + _spacing() );
            columnIndex++;
            if( columnIndex >= _numColumns() ) {
                columnIndex = 0;
                lineIndex++;
            }

        }

        int lineWidth(0);
        columnIndex = 0;
        lineIndex = 0;
        foreach( TaskButton* button, _buttons() )
        {

            Q_UNUSED( button );
            lineWidth += columnWidth[columnIndex];
            columnIndex++;

            // change column
            if( columnIndex >= _numColumns() )
            {
                minimumSize.setWidth( std::max( minimumSize.width(), lineWidth ) );
                minimumSize.setHeight( minimumSize.height()+lineHeight[lineIndex] );

                lineWidth = 0;
                columnIndex = 0;
                lineIndex++;

            }

        }

        minimumSize.setHeight( minimumSize.height() + lineHeight.back() + 2*_margin() );
        minimumSize.setWidth( minimumSize.width() + 2*_margin() );

    }

    // update
    _setMinimumSize( minimumSize );
    return;

}

//_____________________________________________________________
void TaskBarMinimum::_updateMaximumSize( void )
{

    Debug::Throw( "TaskBarMinimum::_updateMaximumSize.\n" );

    // compute maximum size
    const QSize& minimumSize( _minimumSize() );
    QSize maximumSize( _minimumSize() );
    foreach( TaskButton* button, _buttons() )
    {
        maximumSize.setWidth( std::max( maximumSize.width(), minimumSize.width() + button->maximumSize().width() - button->minimumSize().width() ) );
        maximumSize.setHeight( std::max( maximumSize.height(), minimumSize.height() + button->maximumSize().height() - button->minimumSize().height() ) );
    }

    // update
    _setMaximumSize( maximumSize );
    return;

}

//_____________________________________________________________
void TaskBarMinimum::_updateMinimumRectPosition( void )
{

    Debug::Throw( "TaskBarMinimum::_updateMinimumRectPosition.\n" );

    QRect minimum_rect( _minimumRect() );
    QPoint offset;
    const TaskBarAttributes::Layout& layout ( attributes().layout() );
    if( layout & TaskBarAttributes::VERTICAL ) {

        offset.setY( ( _maximumSize().height() - minimum_rect.height() )/2 );
        if( layout & TaskBarAttributes::BOTTOM_RIGHT ) offset.setX( _maximumSize().width() - minimum_rect.width() );
        else if( layout & TaskBarAttributes::CENTER ) offset.setX( ( _maximumSize().width() - minimum_rect.width() )/2 );

    } else if( layout & TaskBarAttributes::HORIZONTAL ) {
        offset.setX( ( _maximumSize().width() - minimum_rect.width() )/2 );
        if( layout & TaskBarAttributes::BOTTOM_RIGHT ) offset.setY( _maximumSize().height() - minimum_rect.height() );
        else if( layout & TaskBarAttributes::CENTER ) offset.setY( ( _maximumSize().height() - minimum_rect.height() )/2 );

    } else  {

        offset.setX( ( _maximumSize().width() - minimum_rect.width() )/2 );
        offset.setY( ( _maximumSize().height() - minimum_rect.height() )/2 );

    }

    minimum_rect.moveTo( offset );
    _setMinimumRect( minimum_rect );

}

//_____________________________________________________________
void TaskBarMinimum::_updateVisibleRect( void )
{

    Debug::Throw( "TaskBarMinimum::_updateVisibleRect.\n" );

    QRect visibleRect;
    const TaskBarAttributes::Layout& layout ( attributes().layout() );
    if( layout & TaskBarAttributes::VERTICAL )
    {

        // width is from minimum size
        visibleRect.setWidth( _minimumSize().width() );

        // get bounding height and y offset
        bool first( true );
        foreach( TaskButton* button, _buttons() )
        {
            visibleRect.setTop( first ? button->rect().top() : std::min( visibleRect.top(), button->rect().top() ) );
            visibleRect.setBottom( std::max( visibleRect.bottom(), button->rect().bottom() ) );
            first = false;
        }

        visibleRect.setTop( visibleRect.top() - _margin() );
        visibleRect.setBottom( visibleRect.bottom() + _margin() );

        if( layout & TaskBarAttributes::BOTTOM_RIGHT ) visibleRect.moveLeft(  _maximumSize().width() - visibleRect.width() );
        else if( layout & TaskBarAttributes::CENTER ) visibleRect.moveLeft( ( _maximumSize().width() - visibleRect.width() )/2 );

    } else if( layout & TaskBarAttributes::HORIZONTAL ) {

        // height is from minimum size
        visibleRect.setHeight( _minimumSize().height() );

        // get bounding height and y offset
        bool first( true );
        foreach( TaskButton* button, _buttons() )
        {
            visibleRect.setLeft( first ? button->rect().left() : std::min( visibleRect.left(), button->rect().left() ) );
            visibleRect.setRight( std::max( visibleRect.right(), button->rect().right() ) );
            first = false;
        }

        visibleRect.setLeft( visibleRect.left() - _margin() );
        visibleRect.setRight( visibleRect.right() + _margin() );

        if( layout & TaskBarAttributes::BOTTOM_RIGHT ) visibleRect.moveTop(  _maximumSize().height() - visibleRect.height() );
        else if( layout & TaskBarAttributes::CENTER ) visibleRect.moveTop( ( _maximumSize().height() - visibleRect.height() )/2 );

    } else {

        visibleRect.setSize( _minimumSize() );
        visibleRect.moveTo( ( _maximumSize().width() - visibleRect.width() )/2, ( _maximumSize().height() - visibleRect.height() )/2 );

    }

    // add offset (for auto-hide)
    visibleRect.translate( _offset() );

    // in case of autoHide, one make sure there is at least
    // a thin band drawn at the bottom, that match the hidden geometry
    if( attributes().flag( TaskBarAttributes::AUTO_HIDE) )
    {

        const TaskBarAttributes::Layout& layout ( attributes().layout() );
        if( layout & TaskBarAttributes::HORIZONTAL )
        {
            if( layout & TaskBarAttributes::TOP_LEFT )  visibleRect.setBottom( std::max(  visibleRect.bottom(), 3 ) );
            else visibleRect.setTop( std::min( visibleRect.top(), height()-3 ) );
        } else {
            if( layout & TaskBarAttributes::TOP_LEFT ) visibleRect.setRight( std::max( visibleRect.right(), 3 ) );
            else visibleRect.setLeft( std::min( visibleRect.left(), width()-3 ) );
        }

    }

    // update blur areas if visibleRect is changed
    if( _setVisibleRect( visibleRect ) && _blurEnabled() )
    {

        static const int margin = 10;
        _updateBlurRegion( _visibleRect().translated( mapTo( window(), QPoint(0,0) ) ).adjusted( margin, margin, -margin, -margin ) );

    }

    return;

}

//_____________________________________________________________
bool TaskBarMinimum::_updateGeometry( Action action )
{

    Debug::Throw() << "TaskBarMinimum::_updateGeometry - action: " << action << endl;

    if( action == NONE ) return false;

    // minimum action
    if( action == MINIMUM )
    {

        setMask( _minimumRect() );
        if( !TRANSPARENCY::CompositeEngine::get().isEnabled() ) _setRootPixmapChanged( true );
        _setAction( NONE );
        return false;

    }

    // enter action
    if( action == ENTER ) {

        if( _rootPixmapChanged() )
        {
            // there is no need to update the root pixmap when composite engine is off,
            // unless during resize events, for which the method is called directly
            if( !TRANSPARENCY::CompositeEngine::get().isEnabled() ) _updateRootPixmap();
            else _setRootPixmapChanged( false );
        }

        if( !TaskBarBase::_updateGeometry( action ) )
        {

            _setAction( NONE );
            setMask( rect() );
            return false;

        } else return true;

    }

    // leave action
    if( action == LEAVE ) {

        if( !_rootPixmapChanged() )
        {
            setMask( _minimumRect() );
            _setRootPixmapChanged( true );
        }

        if( !TaskBarBase::_updateGeometry( action ) )
        {

            _setAction( NONE );
            return false;

        } else return true;

    }

    return false;

}

//_________________________________________________________________
void TaskBarMinimum::_preloadSvg( void ) const
{

    Debug::Throw( "TaskBarMinimum::_preloadSvg.\n" );

    assert( _useSvg() );

    // loop over all possible sizes to preload cache
    QSize maximumSize( maximumSize_ );
    QSize minimumSize( minimumRect_.size() );

    SVG::SvgId::List ids;
    const TaskBarAttributes::Layout& layout( attributes().layout() );
    if( layout & ( TaskBarAttributes::HORIZONTAL ) )
    {

        for( int width = minimumSize.width(); width <= maximumSize.width(); width++ )
        { ids.push_back( QSize( width, minimumSize.height() ) ); }

    } else if( layout & (TaskBarAttributes::HORIZONTAL ) ) {

        for( int height = minimumSize.height(); height <= maximumSize.height(); height++ )
        { ids.push_back( QSize( minimumSize.width(), height ) ); }

    } else ids.push_back( minimumSize );

    // load sizes
    SVG::SvgEngine::get().preload( ids );

    return;

}
